@@ -26,24 +26,21 @@
 /* basic.c ---------------------------------------------------------------- */
 size_t get_goalc_bp (Buffer * bp, Point pt);
 size_t get_goalc (void);
-int backward_char (void);
-int forward_char (void);
+bool previous_line (void);
+bool next_line (void);
+bool backward_char (void);
+bool forward_char (void);
 void gotobob (void);
 void gotoeob (void);
-int next_line (void);
-int ngotodown (size_t n);
-int ngotoup (size_t n);
-int previous_line (void);
 
 /* bind.c ----------------------------------------------------------------- */
-extern Binding root_bindings;
+Function last_command (void);
+void set_this_command (Function cmd);
 size_t do_binding_completion (astr as);
 const char *get_function_by_key (size_t key);
-void process_key (Binding bindings, size_t key);
-Binding init_bindings (void);
+void process_key (size_t key);
 void init_default_bindings (void);
-Function last_command (void);
-void free_bindings (Binding p);
+void free_default_bindings (void);
 
 /* buffer.c --------------------------------------------------------------- */
 #define FIELD(ty, field)                                \
@@ -104,25 +101,26 @@ int completion_try (Completion * cp, astr search, int popup_when_complete);
 void push_mark (void);
 void pop_mark (void);
 void set_mark (void);
-int is_empty_line (void);
-int is_blank_line (void);
+bool is_empty_line (void);
+bool is_blank_line (void);
 int following_char (void);
 int preceding_char (void);
-int bobp (void);
-int eobp (void);
-int bolp (void);
-int eolp (void);
+bool bobp (void);
+bool eobp (void);
+bool bolp (void);
+bool eolp (void);
 void ding (void);
 
 /* eval.c ----------------------------------------------------------------- */
 extern le *leNIL, *leT;
 size_t countNodes (le * branch);
 void leEval (le * list);
-le *execute_with_uniarg (bool undo, int uniarg, int (*forward) (void),
-                         int (*backward) (void));
+le *execute_with_uniarg (bool undo, int uniarg, bool (*forward) (void),
+                         bool (*backward) (void));
 le *execute_function (const char *name, int uniarg);
 Function get_function (const char *name);
 const char *get_function_doc (const char *name);
+int get_function_interactive (const char *name);
 const char *get_function_name (Function p);
 const char *minibuf_read_function_name (const char *fmt, ...);
 void init_eval (void);
@@ -153,6 +151,8 @@ size_t lastkey (void);
 size_t xgetkey (int mode, size_t timeout);
 size_t getkey (void);
 void waitkey (size_t delay);
+void init_getkey (void);
+void free_getkey (void);
 
 /* history.c -------------------------------------------------------------- */
 History *history_new (void);
@@ -187,11 +187,11 @@ void line_replace_text (Line * lp, size_t offset, size_t oldlen,
 int insert_char (int c);
 int insert_char_in_insert_mode (int c);
 bool fill_break_line (void);
-int insert_newline (void);
+bool insert_newline (void);
 void insert_nstring (const char *s, size_t len);
 void insert_astr (astr as);
 void bprintf (const char *fmt, ...);
-int delete_char (void);
+bool delete_char (void);
 void free_registers (void);
 
 /* lisp.c ----------------------------------------------------------------- */
@@ -270,6 +270,22 @@ void recenter (Window * wp);
 void init_search (void);
 void free_search (void);
 
+/* term_curses.c ---------------------------------------------------------- */
+size_t term_buf_len (void);
+void term_init (void);
+void term_close (void);
+void term_suspend (void);
+void term_resume (void);
+void term_move (size_t y, size_t x);
+void term_clrtoeol (void);
+void term_refresh (void);
+void term_clear (void);
+void term_addch (int c);
+void term_attrset (size_t attr);
+void term_beep (void);
+size_t term_xgetkey (int mode, size_t timeout);
+void term_ungetkey (size_t key);
+
 /* term_minibuf.c --------------------------------------------------------- */
 void term_minibuf_write (const char *fmt);
 char *term_minibuf_read (const char *prompt, const char *value, size_t pos,
@@ -286,21 +302,6 @@ void term_tidy (void);
 void term_finish (void);
 void term_addnstr (const char *s, size_t len);
 
-/* term_curses.c ---------------------------------------------------------- */
-void term_init (void);
-void term_close (void);
-void term_suspend (void);
-void term_resume (void);
-void term_move (size_t y, size_t x);
-void term_clrtoeol (void);
-void term_refresh (void);
-void term_clear (void);
-void term_addch (int c);
-void term_attrset (size_t attr);
-void term_beep (void);
-size_t term_xgetkey (int mode, size_t timeout);
-void term_ungetkey (size_t key);
-
 /* undo.c ----------------------------------------------------------------- */
 extern int undo_nosave;
 void undo_start_sequence (void);
