@@ -67,9 +67,10 @@ astr
 agetcwd (void)
 {
   char *s = getcwd (NULL, 0);
+  astr as;
   if (s == NULL)
     s = "";
-  astr as = astr_new_cstr (s);
+  as = astr_new_cstr (s);
   free (s);
   return as;
 }
@@ -424,13 +425,13 @@ If the current buffer now contains an empty file that you just visited
 +*/
 {
   const char *buf = get_buffer_filename (cur_bp);
-  char *base = xstrdup ("");
+  char *base = NULL;
   char *ms;
   astr as = NULL;
 
   if (buf == NULL)
     {
-      as = agetcwd ();
+      as = get_buffer_dir ();
       buf = astr_cstr (as);
     }
   else
@@ -450,8 +451,6 @@ If the current buffer now contains an empty file that you just visited
   free (ms);
   if (as)
     astr_delete (as);
-  else
-    free (buf);
 }
 END_DEFUN
 
@@ -539,7 +538,6 @@ Puts mark after the inserted text.
 +*/
 {
   Buffer *def_bp = ((get_buffer_next (cur_bp) != NULL) ? get_buffer_next (cur_bp) : head_bp);
-  Completion *cp;
 
   if (warn_if_readonly_buffer ())
     return leNIL;
@@ -547,7 +545,7 @@ Puts mark after the inserted text.
   STR_INIT (buffer)
   else
     {
-      cp = make_buffer_completion ();
+      Completion *cp = make_buffer_completion ();
       buffer = minibuf_read_completion ("Insert buffer (default %s): ",
                                         "", cp, NULL, get_buffer_name (def_bp));
       if (buffer == NULL)
@@ -920,9 +918,9 @@ save_buffer (Buffer * bp)
   if (!get_buffer_modified (bp))
     minibuf_write ("(No changes need to be saved)");
   else
+    /* FIXME: Check return value of write_buffer. */
     write_buffer (bp, get_buffer_needname (bp), false, get_buffer_filename (bp),
                   "File to save in: ");
-
   return true;
 }
 
@@ -1027,7 +1025,7 @@ Save some modified file-visiting buffers.  Asks user about each one.
 }
 END_DEFUN
 
-DEFUN ("save-buffers-kill-zile", save_buffers_kill_zile)
+DEFUN ("save-buffers-kill-emacs", save_buffers_kill_emacs)
 /*+
 Offer to save each buffer, then kill this Zile process.
 +*/
