@@ -34,7 +34,7 @@
 #include "extern.h"
 
 
-DEFUN ("suspend-zile", suspend_zile)
+DEFUN ("suspend-emacs", suspend_emacs)
 /*+
 Stop Zile and return to superior process.
 +*/
@@ -74,25 +74,6 @@ With arg, turn Transient Mark mode on if arg is positive, off otherwise.
 }
 END_DEFUN
 
-static char *
-make_buffer_flags (Buffer * bp, int iscurrent)
-{
-  static char buf[4];
-
-  buf[0] = iscurrent ? '.' : ' ';
-  buf[1] = get_buffer_modified (bp) ? '*' : ' ';
-  /*
-   * Display the readonly flag if it is set or the buffer is the
-   * `*Buffer List*' buffer.  (The readonly flag is not set when we are
-   * called, as otherwise we wouldn't be able to write to the
-   * buffer!)
-   */
-  buf[2] = (get_buffer_readonly (bp) || bp == cur_bp) ? '%' : ' ';
-  buf[3] = '\0';
-
-  return buf;
-}
-
 static astr
 make_buffer_modeline (Buffer * bp)
 {
@@ -124,15 +105,14 @@ shorten_string (astr as, int maxlen)
 static void
 print_buf (Buffer * old_bp, Buffer * bp)
 {
-  astr mode = make_buffer_modeline (bp);
-
   if (get_buffer_name (bp)[0] == ' ')
     return;
 
-  bprintf ("%3s %-16s %6u  %-13s",
-           make_buffer_flags (bp, old_bp == bp),
-           get_buffer_name (bp), calculate_buffer_size (bp), astr_cstr (mode));
-  astr_delete (mode);
+  bprintf ("%c%c%c %-19s %6u  %-17s",
+           old_bp == bp ? '.' : ' ',
+           get_buffer_readonly (bp) ? '%' : ' ',
+           get_buffer_modified (bp) ? '*' : ' ',
+           get_buffer_name (bp), calculate_buffer_size (bp), "Fundamental");
   if (get_buffer_filename (bp) != NULL)
     {
       astr shortname = shorten_string (compact_path (astr_new_cstr (get_buffer_filename (bp))), 40);
@@ -201,8 +181,9 @@ write_buffers_list (va_list ap)
   Window *old_wp = va_arg (ap, Window *);
   Buffer *bp;
 
-  bprintf (" MR Buffer           Size    Mode         File\n");
-  bprintf (" -- ------           ----    ----         ----\n");
+  /* FIXME: Underline next line properly. */
+  bprintf ("CRM Buffer                Size  Mode             File\n");
+  bprintf ("--- ------                ----  ----             ----\n");
 
   /* Print buffers. */
   bp = get_window_bp (old_wp);
@@ -234,16 +215,22 @@ The @samp{R} column contains a @samp{%} for buffers that are read-only.
 }
 END_DEFUN
 
-DEFUN ("overwrite-mode", overwrite_mode)
+DEFUN_ARGS ("overwrite-mode", overwrite_mode,
+            INT_OR_UNIARG (arg))
 /*+
-In overwrite mode, printing characters typed in replace existing
-text on a one-for-one basis, rather than pushing it to the right.
-At the end of a line, such characters extend the line.
+Toggle overwrite mode.
+With prefix argument @i{arg}, turn overwrite mode on if @i{arg} is positive,
+otherwise turn it off.  In overwrite mode, printing characters typed
+in replace existing text on a one-for-one basis, rather than pushing
+it to the right.  At the end of a line, such characters extend the line.
+Before a tab, such characters insert until the tab is filled in.
 @kbd{C-q} still inserts characters in overwrite mode; this
 is supposed to make it easier to insert characters when necessary.
 +*/
 {
-  set_buffer_overwrite (cur_bp, !get_buffer_overwrite (cur_bp));
+  INT_OR_UNIARG_INIT (arg);
+  set_buffer_overwrite (cur_bp, lastflag & FLAG_SET_UNIARG ? uniarg > 0 :
+                        !get_buffer_overwrite (cur_bp));
 }
 END_DEFUN
 
@@ -324,9 +311,9 @@ set_mark_interactive (void)
   minibuf_write ("Mark set");
 }
 
-DEFUN ("set-mark-command", set_mark_command)
+DEFUN_NONINTERACTIVE ("set-mark", set_mark)
 /*+
-Set mark at where point is.
+Set this buffer's mark to point.
 +*/
 {
   set_mark_interactive ();
@@ -334,6 +321,15 @@ Set mark at where point is.
 }
 END_DEFUN
 
+DEFUN ("set-mark-command", set_mark_command)
+/*+
+Set the mark where point is.
++*/
+{
+  FUNCALL (set_mark);
+}
+END_DEFUN
+
 DEFUN ("exchange-point-and-mark", exchange_point_and_mark)
 /*+
 Put the mark where point is now, and point where the mark is now.
@@ -420,6 +416,7 @@ You may also type up to 3 octal digits, to insert a character with that code.
 }
 END_DEFUN
 
+/* FIXME: Simplify to one argument, which if KBD_NOKEY is ignored. */
 le *
 universal_argument (int keytype, int xarg)
 {
@@ -432,7 +429,7 @@ universal_argument (int keytype, int xarg)
   if (keytype == KBD_META)
     {
       astr_cpy_cstr (as, "ESC");
-      pushkey ((size_t) (xarg + '0'));
+      pushkey ((size_t) (xarg));
     }
   else
     {
@@ -511,7 +508,7 @@ Repeating @kbd{C-u} without digits or minus sign multiplies the argument
 by 4 each time.
 +*/
 {
-  ok = universal_argument (KBD_CTRL | 'u', 0);
+  ok = universal_argument (0, 0);
 }
 END_DEFUN
 
@@ -714,8 +711,8 @@ END_DEFUN
                           Move through words
 ***********************************************************************/
 #define ISWORDCHAR(c)	(isalnum (c) || c == '$')
-static int
-move_word (int dir, int (*next_char) (void), int (*move_char) (void), int (*at_extreme) (void))
+static bool
+move_word (int dir, int (*next_char) (void), bool (*move_char) (void), bool (*at_extreme) (void))
 {
   int gotword = false;
   for (;;)
@@ -744,13 +741,13 @@ move_word (int dir, int (*next_char) (void), int (*move_char) (void), int (*at_e
   return false;
 }
 
-static int
+static bool
 forward_word (void)
 {
   return move_word (1, following_char, forward_char, eolp);
 }
 
-static int
+static bool
 backward_word (void)
 {
   return move_word (-1, preceding_char, backward_char, bolp);
@@ -892,13 +889,13 @@ move_sexp (int dir)
   return false;
 }
 
-static int
+static bool
 forward_sexp (void)
 {
   return move_sexp (1);
 }
 
-static int
+static bool
 backward_sexp (void)
 {
   return move_sexp (-1);
@@ -946,7 +943,7 @@ astr_append_region (astr s)
 }
 
 static bool
-transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
+transpose_subr (bool (*forward_func) (void), bool (*backward_func) (void))
 {
   Marker *p0 = point_marker (), *m1, *m2;
   astr as1, as2 = NULL;
@@ -1054,7 +1051,7 @@ transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
 }
 
 static le *
-transpose (int uniarg, int (*forward_func) (void), int (*backward_func) (void))
+transpose (int uniarg, bool (*forward_func) (void), bool (*backward_func) (void))
 {
   int uni, ret = true;
 
@@ -1063,7 +1060,7 @@ transpose (int uniarg, int (*forward_func) (void), int (*backward_func) (void))
 
   if (uniarg < 0)
     {
-      int (*tmp_func) (void) = forward_func;
+      bool (*tmp_func) (void) = forward_func;
       forward_func = backward_func;
       backward_func = tmp_func;
       uniarg = -uniarg;
@@ -1153,16 +1150,15 @@ move to with the same argument.
 }
 END_DEFUN
 
+/* FIXME: Use new macro for arg/uniarg */
 DEFUN_ARGS ("forward-line", forward_line,
-            INT_ARG (n))
+            INT_OR_UNIARG (n))
 /*+
 Move N lines forward (backward if N is negative).
 Precisely, if point is on line I, move to the start of line I + N.
 +*/
 {
-  INT_INIT (n)
-  else
-    n = uniarg;
+  INT_OR_UNIARG_INIT (n);
   if (ok == leT)
     {
       FUNCALL (beginning_of_line);
@@ -1172,7 +1168,7 @@ Precisely, if point is on line I, move to the start of line I + N.
 END_DEFUN
 
 static le *
-move_paragraph (int uniarg, int (*forward) (void), int (*backward) (void),
+move_paragraph (int uniarg, bool (*forward) (void), bool (*backward) (void),
                      Function line_extremum)
 {
   if (uniarg < 0)
@@ -1271,8 +1267,6 @@ Fill paragraph at or after point.
          && fill_break_line ())
     ;
 
-  thisflag &= ~FLAG_DONE_CPCN;
-
   set_buffer_pt (cur_bp, get_marker_pt (m));
   free_marker (m);
 
@@ -1280,7 +1274,7 @@ Fill paragraph at or after point.
 }
 END_DEFUN
 
-static int
+static bool
 setcase_word (int rcase)
 {
   size_t i;
@@ -1313,50 +1307,56 @@ setcase_word (int rcase)
   return true;
 }
 
-static int
+static bool
 setcase_word_lowercase (void)
 {
   return setcase_word (case_lower);
 }
 
-DEFUN ("downcase-word", downcase_word)
+DEFUN_ARGS ("downcase-word", downcase_word,
+            INT_OR_UNIARG (arg))
 /*+
-Convert following word (or argument N words) to lower case, moving over.
+Convert following word (or @i{arg} words) to lower case, moving over.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, setcase_word_lowercase, NULL);
+  INT_OR_UNIARG_INIT (arg);
+  ok = execute_with_uniarg (true, arg, setcase_word_lowercase, NULL);
 }
 END_DEFUN
 
-static int
+static bool
 setcase_word_uppercase (void)
 {
   return setcase_word (case_upper);
 }
 
-DEFUN ("upcase-word", upcase_word)
+DEFUN_ARGS ("upcase-word", upcase_word,
+            INT_OR_UNIARG (arg))
 /*+
-Convert following word (or argument N words) to upper case, moving over.
+Convert following word (or @i{arg} words) to upper case, moving over.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, setcase_word_uppercase, NULL);
+  INT_OR_UNIARG_INIT (arg);
+  ok = execute_with_uniarg (true, arg, setcase_word_uppercase, NULL);
 }
 END_DEFUN
 
-static int
+static bool
 setcase_word_capitalize (void)
 {
   return setcase_word (case_capitalized);
 }
 
-DEFUN ("capitalize-word", capitalize_word)
+DEFUN_ARGS ("capitalize-word", capitalize_word,
+            INT_OR_UNIARG (arg))
 /*+
-Capitalize the following word (or argument N words), moving over.
-This gives the word(s) a first character in upper case and the rest
-lower case.
+Capitalize the following word (or @i{arg} words), moving over.
+This gives the word(s) a first character in upper case
+and the rest lower case.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, setcase_word_capitalize, NULL);
+  INT_OR_UNIARG_INIT (arg);
+  ok = execute_with_uniarg (true, arg, setcase_word_capitalize, NULL);
 }
 END_DEFUN
 
@@ -1530,7 +1530,11 @@ says to insert the output in the current buffer.
 }
 END_DEFUN
 
+/* The `start' and `end' arguments are fake, hence their string type,
+   so they can be ignored. */
 DEFUN_ARGS ("shell-command-on-region", shell_command_on_region,
+            STR_ARG (start)
+            STR_ARG (end)
             STR_ARG (cmd)
             BOOL_ARG (insert))
 /*+
@@ -1538,6 +1542,7 @@ Execute string command in inferior shell with region as input.
 Normally display output (if any) in temp buffer `*Shell Command Output*';
 Prefix arg means replace the region with it.  Return the exit code of
 command.
+
 If the command generates output, the output may be displayed
 in the echo area or in a buffer.
 If the output is short enough to display in the echo area, it is shown
@@ -1545,6 +1550,8 @@ there.  Otherwise it is displayed in the buffer `*Shell Command Output*'.
 The output is available in that buffer in both cases.
 +*/
 {
+  STR_INIT (start);
+  STR_INIT (end);
   STR_INIT (cmd)
   else
     cmd = minibuf_read_shell_command ();
