@@ -61,7 +61,7 @@ copy_or_kill_region (bool kill, Region * rp)
 {
   char *p = copy_text_block (get_region_start (rp).n, get_region_start (rp).o, get_region_size (rp));
 
-  if (!(lastflag & FLAG_DONE_KILL))
+  if (last_command () != F_kill_region)
     free_kill_ring ();
   kill_ring_push_nstring (p, get_region_size (rp));
   free (p);
@@ -74,72 +74,113 @@ copy_or_kill_region (bool kill, Region * rp)
         assert (delete_region (rp));
     }
 
-  thisflag |= FLAG_DONE_KILL;
+  set_this_command (F_kill_region);
   deactivate_mark ();
 
   return true;
 }
 
-static int
-kill_line (int literally)
+static bool
+kill_to_bol (void)
 {
+  bool ok = true;
+
+  if (!bolp ())
+    {
+      Region * rp = region_new ();
+      Point pt = get_buffer_pt (cur_bp);
+
+      set_region_size (rp, pt.o);
+      pt.o = 0;
+      set_region_start (rp, pt);
+
+      ok = copy_or_kill_region (true, rp);
+      free (rp);
+    }
+
+  return ok;
+}
+
+static bool
+kill_line (bool literally)
+{
+  bool ok = true;
+  bool start_of_line = bolp ();
+
   if (eobp ())
     {
       minibuf_error ("End of buffer");
       return false;
     }
 
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
+
   if (!eolp ())
     {
       Region * rp = region_new ();
-      bool ret;
 
       set_region_start (rp, get_buffer_pt (cur_bp));
       set_region_size (rp, astr_len (get_line_text (get_buffer_pt (cur_bp).p)) - get_buffer_pt (cur_bp).o);
 
-      ret = copy_or_kill_region (true, rp);
+      ok = copy_or_kill_region (true, rp);
       free (rp);
-
-      if (!ret)
-        return false;
-
-      if (!literally)
-        return true;
     }
 
-  if (!eobp ())
+  if (ok && (literally && start_of_line) && !eobp ())
     {
       if (!FUNCALL (delete_char))
         return false;
 
       kill_ring_push ('\n');
-      thisflag |= FLAG_DONE_KILL;
+      set_this_command (F_kill_region);
     }
 
-  return true;
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
+
+  return ok;
 }
 
-static int
+static bool
 kill_line_literally (void)
 {
   return kill_line (true);
 }
 
-DEFUN ("kill-line", kill_line)
+static bool
+kill_line_backward (void)
+{
+  return previous_line () && kill_line_literally ();
+}
+
+DEFUN_ARGS ("kill-line", kill_line,
+            INT_OR_UNIARG (arg))
 /*+
 Kill the rest of the current line; if no nonblanks there, kill thru newline.
-With prefix argument, kill that many lines from point.
+With prefix argument @i{arg}, kill that many lines from point.
+Negative arguments kill lines backward.
+With zero argument, kills the text before point on the current line.
+
+If `kill-whole-line' is non-nil, then this command kills the whole line
+including its terminating newline, when used at the beginning of a line
+with no argument.
 +*/
 {
-  le * ret = leT;
-
-  if (!(lastflag & FLAG_DONE_KILL))
+  if (last_command () != F_kill_region)
     free_kill_ring ();
 
-  if (!(lastflag & FLAG_SET_UNIARG))
-    kill_line (get_variable_bool ("kill-whole-line"));
+  INT_OR_UNIARG_INIT (arg);
+
+  if (noarg)
+    ok = bool_to_lisp (kill_line (get_variable_bool ("kill-whole-line")));
   else
-    ret = execute_with_uniarg (true, uniarg, kill_line_literally, NULL);
+    {
+      undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
+      if (arg <= 0)
+        ok = bool_to_lisp (kill_to_bol ());
+      if (arg != 0 && ok == leT)
+        ok = execute_with_uniarg (true, arg, kill_line_literally, kill_line_backward);
+      undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
+    }
 
   deactivate_mark ();
 }
@@ -149,13 +190,13 @@ static bool
 copy_or_kill_the_region (bool kill)
 {
   Region * rp = region_new ();
-  bool ret = false;
+  bool ok = false;
 
   if (calculate_the_region (rp))
-    ret = copy_or_kill_region (kill, rp);
+    ok = copy_or_kill_region (kill, rp);
 
   free (rp);
-  return ret;
+  return ok;
 }
 
 DEFUN ("kill-region", kill_region)
@@ -187,7 +228,7 @@ END_DEFUN
 static le *
 kill_text (int uniarg, Function mark_func)
 {
-  if (!(lastflag & FLAG_DONE_KILL))
+  if (last_command () != F_kill_region)
     free_kill_ring ();
 
   if (warn_if_readonly_buffer ())
@@ -200,35 +241,39 @@ kill_text (int uniarg, Function mark_func)
   undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   pop_mark ();
 
-  thisflag |= FLAG_DONE_KILL;
+  set_this_command (F_kill_region);
   minibuf_write ("");		/* Erase "Set mark" message.  */
   return leT;
 }
 
-DEFUN ("kill-word", kill_word)
+DEFUN_ARGS ("kill-word", kill_word,
+            INT_OR_UNIARG (arg))
 /*+
 Kill characters forward until encountering the end of a word.
-With argument, do this that many times.
+With argument @i{arg}, do this that many times.
 +*/
 {
-  ok = kill_text (uniarg, F_mark_word);
+  INT_OR_UNIARG_INIT (arg);
+  ok = kill_text (arg, F_mark_word);
 }
 END_DEFUN
 
-DEFUN ("backward-kill-word", backward_kill_word)
+DEFUN_ARGS ("backward-kill-word", backward_kill_word,
+            INT_OR_UNIARG (arg))
 /*+
 Kill characters backward until encountering the end of a word.
-With argument, do this that many times.
+With argument @i{arg}, do this that many times.
 +*/
 {
-  ok = kill_text (-uniarg, F_mark_word);
+  INT_OR_UNIARG_INIT (arg);
+  ok = kill_text (-arg, F_mark_word);
 }
 END_DEFUN
 
 DEFUN ("kill-sexp", kill_sexp)
 /*+
 Kill the sexp (balanced expression) following the cursor.
-With ARG, kill that many sexps after the cursor.
+With @i{arg}, kill that many sexps after the cursor.
 Negative arg -N means kill N sexps before the cursor.
 +*/
 {
