@@ -208,21 +208,6 @@ insert_char (int c)
   return true;
 }
 
-DEFUN_NONINTERACTIVE_ARGS ("insert-char", insert_char,
-                   STR_ARG (c))
-/*+
-Insert CHARACTER.
-+*/
-{
-  STR_INIT (c);
-  if (strlen (c) > 0)
-    ok = bool_to_lisp (insert_char (*c));
-  else
-    ok = leNIL;
-  STR_FREE (c);
-}
-END_DEFUN
-
 /*
  * Insert a character at the current position in insert mode
  * whatever the current insert mode is.
@@ -254,7 +239,7 @@ insert_expanded_tab (int (*inschr) (int chr))
   undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 
-static int
+static bool
 insert_tab (void)
 {
   if (warn_if_readonly_buffer ())
@@ -282,7 +267,7 @@ END_DEFUN
  * Insert a newline at the current position without moving the cursor.
  * Update markers after point in the split line.
  */
-static int
+static bool
 intercalate_newline (void)
 {
   if (warn_if_readonly_buffer ())
@@ -304,7 +289,7 @@ intercalate_newline (void)
   return true;
 }
 
-int
+bool
 insert_newline (void)
 {
   return intercalate_newline () && forward_char ();
@@ -444,7 +429,7 @@ fill_break_line (void)
   return break_made;
 }
 
-static int
+static bool
 newline (void)
 {
   if (get_buffer_autofill (cur_bp) &&
@@ -488,6 +473,18 @@ insert_nstring (const char *s, size_t len)
   undo_nosave = false;
 }
 
+DEFUN_NONINTERACTIVE_ARGS ("insert", insert,
+                           STR_ARG (arg))
+/*+
+Insert the argument at point.
++*/
+{
+  STR_INIT (arg);
+  insert_nstring (arg, strlen (arg));
+  STR_FREE (arg);
+}
+END_DEFUN
+
 void
 insert_astr (astr as)
 {
@@ -507,7 +504,7 @@ bprintf (const char *fmt, ...)
   free (buf);
 }
 
-int
+bool
 delete_char (void)
 {
   deactivate_mark ();
@@ -547,7 +544,7 @@ delete_char (void)
   return true;
 }
 
-static int
+static bool
 backward_delete_char (void)
 {
   deactivate_mark ();
@@ -562,7 +559,7 @@ backward_delete_char (void)
   return true;
 }
 
-static int
+static bool
 backward_delete_char_overwrite (void)
 {
   if (bolp () || eolp ())
@@ -580,30 +577,34 @@ backward_delete_char_overwrite (void)
     insert_char (' ');
   backward_char ();
 
+  /* FIXME: Remove this line (we've already called insert_char, which
+     sets the modified flag). */
   set_buffer_modified (cur_bp, true);
 
   return true;
 }
 
-DEFUN ("delete-char", delete_char)
+DEFUN_ARGS ("delete-char", delete_char,
+            INT_OR_UNIARG (n))
 /*+
-Delete the following character.
-Join lines if the character is a newline.
+Delete the following @i{n} characters (previous if @i{n} is negative).
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, delete_char, backward_delete_char);
+  INT_OR_UNIARG_INIT (n);
+  ok = execute_with_uniarg (true, n, delete_char, backward_delete_char);
 }
 END_DEFUN
 
-DEFUN ("backward-delete-char", backward_delete_char)
+DEFUN_ARGS ("backward-delete-char", backward_delete_char,
+            INT_OR_UNIARG (n))
 /*+
-Delete the previous character.
-Join lines if the character is a newline.
+Delete the previous @i{n} characters (following if @i{n} is negative).
 +*/
 {
-  int (*forward) (void) = get_buffer_overwrite (cur_bp) ?
+  bool (*forward) (void) = get_buffer_overwrite (cur_bp) ?
     backward_delete_char_overwrite : backward_delete_char;
-  ok = execute_with_uniarg (true, uniarg, forward, delete_char);
+  INT_OR_UNIARG_INIT (n);
+  ok = execute_with_uniarg (true, n, forward, delete_char);
 }
 END_DEFUN
 
